<?php
namespace LiquidElementor\Widgets;

use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Group_Control_Image_Size;
use Elementor\Group_Control_Typography;
use Elementor\Scheme_Color;
use Elementor\Scheme_Typography;
use Elementor\Utils;
use Elementor\Control_Media;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Text_Shadow;
use Elementor\Group_Control_Background;
use Elementor\Repeater;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Elementor heading widget.
 *
 * Elementor widget that displays an eye-catching headlines.
 *
 * @since 1.0.0
 */
class LD_Google_Map extends Widget_Base {

	/**
	 * Get widget name.
	 *
	 * Retrieve heading widget name.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string Widget name.
	 */
	public function get_name() {
		return 'ld_google_map';
	}

	/**
	 * Get widget title.
	 *
	 * Retrieve heading widget title.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string Widget title.
	 */
	public function get_title() {
		return __( 'Liquid Google Maps', 'elementor' );
	}

	/**
	 * Get widget icon.
	 *
	 * Retrieve heading widget icon.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string Widget icon.
	 */
	public function get_icon() {
		return 'eicon-google-maps lqd-element';
	}

	/**
	 * Get widget categories.
	 *
	 * Retrieve the list of categories the heading widget belongs to.
	 *
	 * Used to determine where to display the widget in the editor.
	 *
	 * @since 2.0.0
	 * @access public
	 *
	 * @return array Widget categories.
	 */
	public function get_categories() {
		return [ 'hub-core' ];
	}

	/**
	 * Get widget keywords.
	 *
	 * Retrieve the list of keywords the widget belongs to.
	 *
	 * @since 2.1.0
	 * @access public
	 *
	 * @return array Widget keywords.
	 */
	public function get_keywords() {
		return [ 'map', 'google' ];
	}

	/**
	 * Retrieve the list of scripts the counter widget depended on.
	 *
	 * Used to set scripts dependencies required to run the widget.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return array Widget scripts dependencies.
	 */
	public function get_script_depends() {
		return [ 'google-maps-api', 'gsap' ];
	}

	/**
	 * Register heading widget controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	protected function _register_controls() {

		$this->start_controls_section(
			'map_section',
			[
				'label' => __( 'Google Map', 'hub-elementor-addons' ),
			]
		);
		
		$this->add_control(
            'style',
            [
                'label' => __( 'Map Style', 'hub-elementor-addons' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'wy',
                'options' => [
                    'assassinsCreedIV' => __( 'Assassins Creed IV', 'hub-elementor-addons' ),
                    'blueEssence' => __( 'Blue Essence', 'hub-elementor-addons' ),
                    'classic' => __( 'Classic', 'hub-elementor-addons' ),
                    'lightMonochrome' => __( 'Light Monochrome', 'hub-elementor-addons' ),
                    'unsaturatedBrowns' => __( 'Unsaturated Browns', 'hub-elementor-addons' ),
                    'wy' => __( 'WY', 'hub-elementor-addons' ),
                    'evenLighter' => __( 'Even Lighter', 'hub-elementor-addons' ),
                    'shadesOfGray' => __( 'Shades of Gray', 'hub-elementor-addons' ),
                ],
            ]
        );

        $this->add_control(
			'address',
			[
				'label' => __( 'Address', 'hub-elementor-addons' ),
				'type' => Controls_Manager::TEXTAREA,
				'rows' => 4,
				'default' => '7420 Shore Rd, Brooklyn, NY 11209, USA',
			]
		);

        $this->add_responsive_control(
			'map_height',
			[
				'label' => __( 'Height', 'hub-elementor-addons' ),
				'type' => Controls_Manager::TEXT,
				'default' => '550px',
				'selectors' => [
					'{{WRAPPER}} .ld-gmap-container' => 'height: {{VALUE}}',
				],
			]
		);

		$this->add_control(
			'map_marker',
			[
				'label' => __( 'Marker', 'hub-elementor-addons' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'default',
				'separator' => 'before',
				'options' => [
					'default' => __( 'Default', 'hub-elementor-addons' ),
					'image' => __( 'Image', 'hub-elementor-addons' ),
					'html' => __( 'Animated Cricles', 'hub-elementor-addons' ),
				],
			]
		);

		$this->add_control(
			'color_marker',
			[
				'label' => __( 'Marker Color', 'hub-elementor-addons' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [ 
					'{{WRAPPER}} .lqd-custom-map-marker, .lqd-custom-map-marker div' => 'background-color: {{VALUE}}',
				],
				'condition' => [
					'map_marker' => 'html',
				]
			]
		);

		$this->add_control(
			'custom_marker',
			[
				'label' => __( 'Custom Marker', 'hub-elementor-addons' ),
				'type' => Controls_Manager::MEDIA,
				'condition' => [
					'map_marker' => 'image',
				]
			]
		);

		$this->add_control(
			'multiple_markers',
			[
				'label' => __( 'Multiple markers?', 'hub-elementor-addons' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => __( 'Yes', 'hub-elementor-addons' ),
				'label_off' => __( 'No', 'hub-elementor-addons' ),
				'return_value' => 'yes',
				'default' => 'no',
			]
		);

		$repeater = new Repeater();

		$repeater->add_control(
			'title',
			[
				'label' => __( 'Title', 'hub-elementor-addons' ),
				'type' => Controls_Manager::TEXT,
				'label_block' => true,
				'dynamic' => [
					'active' => true,
				],
			]
		);
		$repeater->add_control(
			'lat',
			[
				'label' => __( 'Latitude', 'hub-elementor-addons' ),
				'description' => esc_html__( 'Marker Latitude', 'hub-elementor-addons' ),
				'type' => Controls_Manager::TEXT,
				'dynamic' => [
					'active' => true,
				],
				'label_block' => true,
			]
		);
		$repeater->add_control(
			'long',
			[
				'label' => __( 'Longitude', 'hub-elementor-addons' ),
				'description' => esc_html__( 'Marker Longitude', 'hub-elementor-addons' ),
				'type' => Controls_Manager::TEXT,
				'dynamic' => [
					'active' => true,
				],
				'label_block' => true,
			]
		);
		$this->add_control(
			'marker_coordinates',
			[
				'label' => __( 'Marker\'s coordinates', 'hub-elementor-addons' ),
				'type' => Controls_Manager::REPEATER,
				'fields' => $repeater->get_controls(),
				'title_field' => '{{{ title }}}',
				'condition' => [
					'multiple_markers' => 'yes'
				]
			]
		);

		$this->add_control(
			'map_type',
			[
				'label'  => __( 'Map Type', 'hub-elementor-addons' ),
				'type'   => Controls_Manager::SELECT,
				'options' => [
					'roadmap'   => esc_html__( 'Roadmap', 'hub-elementor-addons' ),
					'satellite' => esc_html__( 'Satellite', 'hub-elementor-addons' ),
					'hybrid'    => esc_html__( 'Hybrid', 'hub-elementor-addons' ),
					'terrain'   => esc_Html__( 'Terrain', 'hub-elementor-addons' ),
				],
				'default' => 'roadmap',
				'separator' => 'before',
			]
		);

		$this->add_control(
			'zoom',
			[
				'label' => __( 'Zoom', 'elementor' ),
				'type' => Controls_Manager::SLIDER,
				'default' => [
					'size' => 16,
				],
				'range' => [
					'px' => [
						'min' => 1,
						'max' => 20,
					],
				],
				'separator' => 'before',
			]
		);

		$this->add_control(
			'map_controls',
			[
				'label'  => __( 'Enable/Disable controls', 'hub-elementor-addons' ),
				'type'   => Controls_Manager::SELECT2,
				'label_block' => true,
				'multiple' => true,
				'options' => [
					'fullscreenControl' => __( 'Fullscreen', 'hub-elementor-addons' ),
					'panControl'        => __( 'Pan', 'hub-elementor-addons' ),
					'rotateControl'     => __( 'Rotate', 'hub-elementor-addons' ),
					'scaleControl'      => __( 'Scale', 'hub-elementor-addons' ),
					'scrollwheel'       => __( 'Scrollwheel', 'hub-elementor-addons' ),
					'streetViewControl' => __( 'Street View', 'hub-elementor-addons' ),
					'zoomControl'       => __( 'Zoom', 'hub-elementor-addons' ),
				],
				'default' => [
					'zoomControl'
				],
				'separator' => 'before',
			]
		);

		$this->end_controls_section();

	}

	protected function get_marker() {

		$map_marker = $this->get_settings_for_display('map_marker');

		if( empty( $map_marker ) || 'html' === $map_marker ) {
			return '';
		}

		if ( 'image' === $map_marker && isset( $this->get_settings_for_display('custom_marker')['url'] ) ) {
			return $this->get_settings_for_display('custom_marker')['url'];
		}

	}

	protected function get_coordinates() {

		$items = array();
		
		if( 'no' === $this->get_settings_for_display('multiple_markers') ) {
			return;
		}
		
		$items = is_array($this->get_settings_for_display('marker_coordinates')) ? array_filter( $this->get_settings_for_display('marker_coordinates') ) : null;
		
		if( empty( $items ) ) {
			return;
		}

		$data = array();

		foreach( $items as $item ) {
			$data[] = array( ''. esc_attr( $item['lat'] ) . '', '' . esc_attr( $item['long'] ) . '' );
		}

		return $data;

	}

	/**
	 * Render heading widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	protected function render() {
		
		$settings = $this->get_settings_for_display();

		if ( 0 === absint( $settings['zoom']['size'] ) ) {
			$settings['zoom']['size'] = 14;
		}
		
		$options = array(
			'style' => $settings['style'],
			'address' => $settings['address'],
			'marker_style' => $settings['map_marker'],
			'markers' => $this->get_coordinates(),
			'map'     => array(
				'zoom'      => $settings['zoom']['size'] ? intval( $settings['zoom']['size'] ) : 14,
				'mapTypeId' => $settings['map_type']
			)
		);

		if ( ! empty( $this->get_marker() ) ) {
			$options['marker'] = $this->get_marker();
		}

		// Map Controls
		$map_controls = $settings['map_controls'];
		
		if( $map_controls ) {
			$map = array();
			foreach( $map_controls as $control ) {
				$options['map'][ $control ] = true;
			}
		}
		
		?>

			<div class="ld-gmap-container">
				<div class="ld-gmap" data-plugin-map="true" data-plugin-options='<?php echo wp_json_encode( $options ) ?>'></div>
			</div>

		<?php

	}

}
