<?php
namespace LiquidElementor\Widgets;

use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Group_Control_Image_Size;
use Elementor\Group_Control_Typography;
use Elementor\Scheme_Color;
use Elementor\Scheme_Typography;
use Elementor\Utils;
use Elementor\Control_Media;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Text_Shadow;
use Elementor\Group_Control_Background;
use Elementor\Repeater;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Elementor heading widget.
 *
 * Elementor widget that displays an eye-catching headlines.
 *
 * @since 1.0.1
 */
class LD_Slideshow extends Widget_Base {

	/**
	 * Get widget name.
	 *
	 * Retrieve heading widget name.
	 *
	 * @since 1.0.1
	 * @access public
	 *
	 * @return string Widget name.
	 */
	public function get_name() {
		return 'ld_slideshow';
	}

	/**
	 * Get widget title.
	 *
	 * Retrieve heading widget title.
	 *
	 * @since 1.0.1
	 * @access public
	 *
	 * @return string Widget title.
	 */
	public function get_title() {
		return __( 'Liquid Slideshow', 'hub-elementor-addons' );
	}

	/**
	 * Get widget icon.
	 *
	 * Retrieve heading widget icon.
	 *
	 * @since 1.0.1
	 * @access public
	 *
	 * @return string Widget icon.
	 */
	public function get_icon() {
		return 'eicon-slideshow lqd-element';
	}

	/**
	 * Get widget categories.
	 *
	 * Retrieve the list of categories the heading widget belongs to.
	 *
	 * Used to determine where to display the widget in the editor.
	 *
	 * @since 1.0.1
	 * @access public
	 *
	 * @return array Widget categories.
	 */
	public function get_categories() {
		return [ 'hub-core' ];
	}

	/**
	 * Get widget keywords.
	 *
	 * Retrieve the list of keywords the widget belongs to.
	 *
	 * @since 1.0.1
	 * @access public
	 *
	 * @return array Widget keywords.
	 */
	public function get_keywords() {
		return [ 'slide', 'slider', 'carousel' ];
	}

	/**
	 * Retrieve the list of scripts the counter widget depended on.
	 *
	 * Used to set scripts dependencies required to run the widget.
	 *
	 * @since 1.0.1
	 * @access public
	 *
	 * @return array Widget scripts dependencies.
	 */
	public function get_script_depends() {
		return [ 'flickity' ];
	}

	/**
	 * Register heading widget controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 1.0.1
	 * @access protected
	 */
	protected function _register_controls() {

		$this->start_controls_section(
			'general_section',
			[
				'label' => __( 'General', 'hub-elementor-addons' ),
				'tab' => Controls_Manager::TAB_CONTENT,
			]
		);

		$this->add_responsive_control(
			'columns',
			[
				'label' => __( 'Number of Columns', 'hub-elementor-addons' ),
				'type' => Controls_Manager::SLIDER,
				'range' => [
					'px' => [
						'min' => 1,
						'max' => 10,
						'step' => 0.5,
					],
				],
			'devices' => [ 'desktop', 'tablet', 'mobile' ],
				'desktop_default' => [
					'size' => 3,
					'unit' => 'px',
				],
				'tablet_default' => [
					'size' => 2,
					'unit' => 'px',
				],
				'mobile_default' => [
					'size' => 1,
					'unit' => 'px',
				],
				'selectors' => [
					'{{WRAPPER}} .carousel-item' => 'width: calc(100% / {{SIZE}}); flex-basis: calc(100% / {{SIZE}});',
				],
			]
		);

		$repeater = new Repeater();

		$repeater->add_control(
			'title', [
				'label' => __( 'Title', 'hub-elementor-addons' ),
				'type' => Controls_Manager::TEXT,
				'default' => __( 'Title' , 'hub-elementor-addons' ),
				'label_block' => true,
			]
		);
		
		$repeater->add_control(
			'description', [
				'label' => __( 'Description', 'hub-elementor-addons' ),
				'type' => Controls_Manager::TEXT,
				'default' => __( 'Description' , 'hub-elementor-addons' ),
				'label_block' => true,
			]
		);

		$repeater->add_control(
			'image',
			[
				'label' => __( 'Image', 'hub-elementor-addons' ),
				'type' => Controls_Manager::MEDIA,
				'default' => [
					'url' => Utils::get_placeholder_image_src(),
				],
			]
		);

		$repeater->add_control(
			'url',
			[
				'label' => __( 'URL (Link)', 'hub-elementor-addons' ),
				'type' => Controls_Manager::URL,
				'placeholder' => __( 'https://your-link.com', 'hub-elementor-addons' ),
				'show_external' => true,
				'default' => [
					'url' => '',
				],
			]
		);

		$repeater->add_control(
			'btn_label', [
				'label' => __( 'Button Label', 'hub-elementor-addons' ),
				'type' => Controls_Manager::TEXT,
				'default' => __( 'Button' , 'hub-elementor-addons' ),
				'label_block' => true,
			]
		);

		$this->add_control(
			'identities',
			[
				'label' => __( 'Slides', 'hub-elementor-addons' ),
				'type' => Controls_Manager::REPEATER,
				'fields' => $repeater->get_controls(),
				'default' => [
					[],[],
				],
				'title_field' => '{{{ title }}}',
			]
		);

		$this->end_controls_section();

		$this->start_controls_section(
			'style_section',
			[
				'label' => __( 'General', 'hub-elementor-addons' ),
				'tab' => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'title_typography',
				'label' => __( 'Title Typography', 'hub-elementor-addons' ),
				'selector' => '{{WRAPPER}} .lqd-slsh h2',
			]
		);

		$this->add_control(
			'title_color',
			[
				'label' => __( 'Title Color', 'hub-elementor-addons' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .lqd-slsh h2' => 'color: {{VALUE}}',
				],
				'separator' => 'after',
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'desc_typography',
				'label' => __( 'Description Typography', 'hub-elementor-addons' ),
				'selector' => '{{WRAPPER}} .lqd-slsh p',
			]
		);

		$this->add_control(
			'desc_color',
			[
				'label' => __( 'Description Color', 'hub-elementor-addons' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .lqd-slsh p' => 'color: {{VALUE}}',
				],
				'separator' => 'after',
			]
		);
		
		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'button_typography',
				'label' => __( 'Button Typography', 'hub-elementor-addons' ),
				'selector' => '{{WRAPPER}} .lqd-slsh .btn',
			]
		);

		$this->add_control(
			'button_color',
			[
				'label' => __( 'Button Color', 'hub-elementor-addons' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .lqd-slsh .btn' => 'color: {{VALUE}}',
				],
				'separator' => 'after',
			]
		);

		$this->add_control(
			'overlay_bg',
			[
				'label' => __( 'Overlay Background', 'hub-elementor-addons' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .lqd-slsh-overlay-bg' => 'background: {{VALUE}}',
				],
			]
		);

		$this->end_controls_section();

	}

	/**
	 * Render heading widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.0.1
	 * @access protected
	 */
	protected function render() {
		
		$settings = $this->get_settings_for_display();

		?>

		<div id="lqd-slsh-<?php echo $this->get_id() ?>" class="lqd-slsh"  data-active-onhover="true" data-active-onhover-options='{ "triggers": ".carousel-container .carousel-item", "targets": ".lqd-slsh-img-container .lqd-slsh-img-full" }'>

		<div class="carousel-container carousel-nav-floated carousel-nav-top carousel-nav-lg carousel-nav-square carousel-nav-solid">

			<div class="carousel-items row pos-rel z-index-2 mx-0" data-lqd-flickity='{ "prevNextButtons": true, "wrapAround": true, "equalHeightCells": true, "groupCells": false, "navArrow": 6 }'>

				<?php 
				if ( $settings['identities'] ) {
					
					$i = 1; 
					foreach ( $settings['identities'] as $slide ) { 
					
				?>
				
				<div class="carousel-item col-md-3 col-xs-6 px-0">

					<div
					class="lqd-slsh-item pos-rel h-pt-150"
					data-slideelement-onhover="true"
					data-slideelement-options='{ "visibleElement": "h2 .lqd-lines", "hiddenElement": "p .lqd-lines, .lqd-slsh-btn", "waitForSplitText": true }'>
						
						<?php if( !empty( $slide['url']['url'] ) ) { ?>
							<a target="_blank" href="<?php echo esc_url( $slide['url']['url'] ); ?>" class="liquid-overlay-link z-index-3"></a>
						<?php } ?>
						
						<?php if( isset( $slide['image']['url'] ) ) { ?>
						<div class="lqd-slsh-img lqd-overlay">
							<figure class="w-100">
								<?php
									
									$alt    = get_post_meta( $slide['image']['id'], '_wp_attachment_image_alt', true );
									$image  = wp_get_attachment_image( $slide['image']['id'], 'full', false, array( 'class' => 'lqd-overlay objfit-cover objpos-center', 'alt' => esc_attr( $alt ) ) );
									
									echo $image;

								?>
							</figure>
							<div class="lqd-slsh-overlay-bg lqd-overlay"></div>
						</div>
						<?php } ?>
						<div class="lqd-slsh-content lqd-overlay flex-column justify-content-end px-3 py-4 px-md-6 py-md-5">
							<div class="lqd-slsh-content-inner pos-rel">
								<h2
								class="lqd-slsh-h-org my-0"
								data-split-text="true"
								data-split-options='{ "type": "lines" }'
								data-fittext="true"
								data-fittext-options='{ "maxFontSize": "currentFontSize" }'><?php echo esc_html( $slide['title'] ); ?></h2>
								
								<?php if( !empty( $slide['description'] ) ) { ?>
								<p
								class="mt-3"
								data-split-text="true"
								data-split-options='{ "type": "lines" }'><?php echo $slide['description'] ?></p><?php } ?>
								
								<?php if( isset( $slide['url']['url'] ) ) { ?>
								<div class="lqd-slsh-btn mt-4">
									<a href="<?php echo esc_url( $slide['url']['url'] ) ?>" class="btn btn-sm btn-naked btn-hover-swp">
										<span>
											<span class="btn-txt"><?php echo $slide['btn_label'] ?></span>
											<span class="btn-icon">
												<i class="lqd-icn-ess icon-md-arrow-round-forward"></i>
											</span>
											<span class="btn-icon">
												<i class="lqd-icn-ess icon-md-arrow-round-forward"></i>
											</span>
										</span>
									</a>
								</div>
								<?php } ?>

								
							</div>
						</div>

					</div>

				</div>

				<?php $i++; } 
				}
				?>
				
				
			</div>

			<div class="lqd-slsh-img-container lqd-overlay">
			
			<?php foreach ( $settings['identities'] as $slide ) { ?>

				<figure class="lqd-slsh-img-full lqd-overlay">
				<?php
				
				if( isset( $slide['image']['url'] ) ) { 
					
					$alt    = get_post_meta( $slide['image']['id'], '_wp_attachment_image_alt', true );
					$image  = wp_get_attachment_image( $slide['image']['id'], 'full', false, array( 'class' => 'objfit-cover objpos-center lqd-overlay', 'alt' => esc_attr( $alt ) ) );
					
					echo $image;

				}

				?>
				</figure>
				
			<?php } ?>


			</div>

		</div>

		</div>

		<?php
		
	}

}
